/*---------------------------------------------------------------------*/
/* --- Web: www.STCAI.com ---------------------------------------------*/
/*---------------------------------------------------------------------*/

#include "usb.h"
#include "usb_req_class.h"

#include "main.h"


BYTE bHidIdle;
BYTE bKeyCode = 0;
BYTE bKeyDebounce = 0;
BOOL fKeyOK;

sbit LED_NUM        = P4^3;
sbit LED_CAPS       = P4^2;
sbit LED_SCROLL     = P4^1;

void usb_req_class()
{
    switch (Setup.bRequest)
    {
    case GET_REPORT:
        usb_get_report();
        break;
    case SET_REPORT:
        usb_set_report();
        break;
    case GET_IDLE:
        usb_get_idle();
        break;
    case SET_IDLE:
        usb_set_idle();
        break;
    case GET_PROTOCOL:
        usb_get_protocol();
        break;
    case SET_PROTOCOL:
        usb_set_protocol();
        break;
    default:
        usb_setup_stall();
        return;
    }
}

void usb_get_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_in();
}

void usb_set_report()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = UsbBuffer;
    Ep0State.wSize = Setup.wLength;

    usb_setup_out();
}

void usb_get_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (IN_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    Ep0State.pData = &bHidIdle;
    Ep0State.wSize = 1;

    usb_setup_in();
}

void usb_set_idle()
{
    if ((DeviceState != DEVSTATE_CONFIGURED) ||
        (Setup.bmRequestType != (OUT_DIRECT | CLASS_REQUEST | INTERFACE_RECIPIENT)))
    {
        usb_setup_stall();
        return;
    }

    bHidIdle = Setup.wValueH;

    usb_setup_status();
}

void usb_get_protocol()
{
    usb_setup_stall();
}

void usb_set_protocol()
{
    usb_setup_stall();
}

void usb_class_out()
{
    BYTE led;
    
    if (usb_bulk_intr_out(UsbBuffer, 1) == 1)
    {
        P4M0 &= ~0x0f;
        P4M1 &= ~0x0f;
        P40 = 0;
        
        led = UsbBuffer[0];
        LED_NUM = !(led & 0x01);
        LED_CAPS = !(led & 0x02);
        LED_SCROLL = !(led & 0x04);
    }
}

void usb_class_in(BYTE modifier,BYTE keycode[6])
{
    BYTE key[8];
    BYTE i;
    
    if (DeviceState != DEVSTATE_CONFIGURED)
        return;
fKeyOK = 1;
    if (!UsbInBusy && fKeyOK)
    {
        fKeyOK = 0;

        key[0] = modifier;
        key[1] = 0;
        key[2] = 0;
        key[3] = 0;
        key[4] = 0;
        key[5] = 0;
        key[6] = 0;
        key[7] = 0;
        for(i = 2;i<8;i++)
        {
            key[i] = keycode[i-2];
        }
        switch (bKeyCode)
        {
        case 0xfe:  key[2] = 0x1e; break;
        case 0xfd:  key[2] = 0x1f; break;
        case 0xfb:  key[2] = 0x20; break;
        case 0xf7:  key[2] = 0x21; break;
        case 0xef:  key[2] = 0x22; break;
        case 0xdf:  key[2] = 0x23; break;
        case 0xbf:  key[2] = 0x24; break;
        case 0x7f:  key[2] = 0x25; break;
        }
        
        IE2 &= ~EUSB;
        UsbInBusy = 1;
        usb_write_reg(INDEX, 1);
        for (i=0; i<8; i++)
        {
            usb_write_reg(FIFO1, key[i]);
        }
        usb_write_reg(INCSR1, INIPRDY);
        IE2 |= EUSB;
    }
}

void scan_key()
{
    BYTE keycode[6] = {0};
    BYTE keycode1[6] = {0};
    if(P21 == 0)
    {
        Delay_ms(100);	//@24.000MHz
        if(P21 == 0)
        {
          
            keycode[0] = 0x04;
            usb_class_in(KEYBOARD_MODIFIER_LEFTSHIFT,keycode);
               Delay_ms(20);	//@24.000MHz
            
            usb_class_in(KEYBOARD_MODIFIER_NONE,keycode1);
        }
    }else
    {
      //  bKeyCode = 0;
      //  fKeyOK = 0;
    }
    if(P22 == 0)
    {
        Delay_ms(100);	//@24.000MHz
        if(P22 == 0)
        {

            keycode[0] = 0x05;
            usb_class_in(KEYBOARD_MODIFIER_NONE,keycode);
            Delay_ms(20);
            usb_class_in(KEYBOARD_MODIFIER_NONE,keycode1);
        }
    }else
    {
      //  bKeyCode = 0;
      //  fKeyOK = 0;
    }
    if(P20 == 0)
    {
        //Delay_ms(100);	//@24.000MHz
        if(P20 == 0)
        {

            P00 = 0;
            Delay_ms(100);	//@24.000MHz
            P00 = 1;
            Delay_ms(100);	//@24.000MHz
        }
    }else
    {
      //  bKeyCode = 0;
      //  fKeyOK = 0;
    }
    
    
}

